package com.cusc.nirvana.user.rnr.openapi.interceptor;

import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.support.spring.FastJsonHttpMessageConverter;
import com.cusc.nirvana.user.rnr.openapi.config.OpenapiConfig;
import com.cusc.nirvana.user.rnr.openapi.constants.AuthConstant;
import com.cusc.nirvana.user.rnr.openapi.dto.OrganizationRespDTO;
import com.cusc.nirvana.user.rnr.openapi.dto.PagerDTO;
import com.cusc.nirvana.user.rnr.openapi.exception.RnrOpenApiException;
import com.cusc.nirvana.user.rnr.openapi.service.IOrganizationService;
import com.cusc.nirvana.user.rnr.openapi.util.ThreadLocalUtil;
import com.cusc.nirvana.user.util.crypt.Sm4Util;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.MethodParameter;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpInputMessage;
import org.springframework.http.converter.HttpMessageConverter;
import org.springframework.web.bind.annotation.RestControllerAdvice;

import java.io.*;
import java.lang.reflect.Type;
import java.util.List;

@Slf4j
@RestControllerAdvice
public class RequestBodyDecodeAdvice extends RequestBodyAdviceAdapter {

    @Autowired
    private IOrganizationService organizationService;

    @Autowired
    private OpenapiConfig openapiConfig;

    @Override
    public boolean supports(MethodParameter methodParameter, Type targetType,
                            Class<? extends HttpMessageConverter<?>> converterType) {
        /**
         * 系统使用的是FastJson作为json数据的Http消息转换器
         */
        return FastJsonHttpMessageConverter.class.isAssignableFrom(converterType);
    }

    @Override
    public HttpInputMessage beforeBodyRead(HttpInputMessage inputMessage, MethodParameter parameter, Type targetType,
                                           Class<? extends HttpMessageConverter<?>> converterType) throws IOException {

        // 读取加密的请求体 todo 后续要进行加白名单功能
        if (!"decode".equals(parameter.getMethod().getName()) && !"encode".equals(parameter.getMethod().getName())
                && !"liveness".equals(parameter.getMethod().getName())) {
//            byte[] body = new byte[inputMessage.getBody().available()];
//            inputMessage.getBody().read(body);
//            String bodySource = new String(body, StandardCharsets.UTF_8);
            String bodySource = getStringByInputStream(inputMessage.getBody());
            if (StringUtils.isNotEmpty(bodySource)) {
                if (bodySource.contains("\"")) {
                    bodySource = bodySource.replace("\"", "").trim();
                }
                try {
                    // 解密
                    String bodyStr = Sm4Util.decryptEcbPaddingString(openapiConfig.getDataSecret(), bodySource);

                    //从请求头获取开放平台租户ID
                    List<String> openTenantId = inputMessage.getHeaders().get(AuthConstant.HEADER_TENANT_ID);
                    JSONObject param = JSONObject.parseObject(bodyStr);
                    if (null == param) {
                        throw new RnrOpenApiException("解密body时候出错,请检查租户相关秘钥配置后再试");
                    }
                    String orgId = (String) param.get(AuthConstant.BODY_ORG_ID);
                    if (StringUtils.isEmpty(orgId) && org.apache.commons.collections4.CollectionUtils.isNotEmpty(
                            openTenantId)) {
                        PagerDTO pagerDTO = new PagerDTO();
                        try {
                            List<OrganizationRespDTO> orgList =
                                    organizationService.getOrgListByPage(openTenantId.get(0), pagerDTO);
                            if (!org.apache.commons.collections4.CollectionUtils.isEmpty(orgList)) {
                                orgId = orgList.get(0).getUuid();
                                param.put(AuthConstant.BODY_ORG_ID, orgId);
                                ThreadLocalUtil.set(AuthConstant.BODY_ORG_ID, orgId);
                            }
                        } catch (Exception e) {
                            log.error("getOrgListByPage-error", e);
                        }
                    }


                    byte[] body = param.toJSONString().getBytes();

                    // 使用解密后的数据，构造新的读取流
                    InputStream rawInputStream = new ByteArrayInputStream(body);
                    return new HttpInputMessage() {
                        @Override
                        public HttpHeaders getHeaders() {
                            return inputMessage.getHeaders();
                        }

                        @Override
                        public InputStream getBody() throws IOException {
                            return rawInputStream;
                        }
                    };
                } catch (Exception e) {
                    log.error("解密body失败", e);
                    throw new RnrOpenApiException("解密body失败,请检查租户相关秘钥配置后再试");
                }
            }
        }
        return inputMessage;
    }

    private String getStringByInputStream(InputStream inputStream) {
        BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(inputStream));
        try {
            StringBuilder result = new StringBuilder();
            String line;
            while ((line = bufferedReader.readLine()) != null) {
                result.append(line);
            }
            return result.toString();
        } catch (Exception e) {
            try {
                inputStream.close();
                bufferedReader.close();
            } catch (Exception e1) {
                log.error("body-inputstream转String异常", e);
            }
        }
        return null;
    }
}

