package com.cusc.nirvana.user.rnr.openapi.interceptor;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.serializer.SerializerFeature;
import com.cusc.nirvana.common.result.Response;
import com.cusc.nirvana.user.rnr.openapi.config.OpenapiConfig;
import com.cusc.nirvana.user.util.crypt.Sm4Util;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.MethodParameter;
import org.springframework.http.MediaType;
import org.springframework.http.server.ServerHttpRequest;
import org.springframework.http.server.ServerHttpResponse;
import org.springframework.web.bind.annotation.RestControllerAdvice;
import org.springframework.web.servlet.mvc.method.annotation.ResponseBodyAdvice;

@Slf4j
@RestControllerAdvice
public class ResponseBodyEncodeAdvice implements ResponseBodyAdvice {

    @Autowired
    OpenapiConfig openapiConfig;

    @Override
    public boolean supports(MethodParameter returnType, Class converterType) {
        return true;
    }

    @Override
    public Object beforeBodyWrite(Object body, MethodParameter returnType, MediaType selectedContentType, Class selectedConverterType, ServerHttpRequest request, ServerHttpResponse response) {
        // 读取加密的请求体
        if (body instanceof Response) {
            Response rnrResponse = (Response) body;
            if (rnrResponse.isSuccess() && !"decode".equals(returnType.getMethod().getName()) && !"encode".equals(returnType.getMethod().getName())) {

                try {
                    String bodyStr = JSON.toJSONString(rnrResponse.getData(), SerializerFeature.MapSortField);
                    String encodeData = Sm4Util.encryptEcbPaddingHexString(openapiConfig.getDataSecret(), bodyStr);
                    rnrResponse.setData(encodeData);
                } catch (Exception e) {
                    log.error("获取租户秘钥信息失败或解密失败-error", e);
                }
            }

            return rnrResponse;
        } else {
            Response resp = (Response) body;
            if (resp.isSuccess() && !"decode".equals(returnType.getMethod().getName()) && !"encode".equals(returnType.getMethod().getName())) {

                try {
                    String bodyStr = JSON.toJSONString(resp.getData(), SerializerFeature.MapSortField);
                    String encodeData = Sm4Util.encryptEcbPaddingHexString(openapiConfig.getDataSecret(), bodyStr);
                    resp.setData(encodeData);
                } catch (Exception e) {
                    log.error("获取租户秘钥信息失败或解密失败-error", e);
                }
            }
            return resp;
        }
    }
}

