package com.cusc.nirvana.user.rnr.openapi.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.alibaba.nacos.common.utils.MapUtils;
import com.cusc.nirvana.common.result.Response;
import com.cusc.nirvana.user.rnr.mg.client.MgRnrCardInfoClient;
import com.cusc.nirvana.user.rnr.mg.client.MgRnrInfoClient;
import com.cusc.nirvana.user.rnr.mg.client.MgVehicleCompanyClient;
import com.cusc.nirvana.user.rnr.mg.constants.CompanyTypeEnum;
import com.cusc.nirvana.user.rnr.mg.constants.RnrBizzTypeEnum;
import com.cusc.nirvana.user.rnr.mg.constants.RnrStatus;
import com.cusc.nirvana.user.rnr.mg.dto.MgRnrCardInfoDTO;
import com.cusc.nirvana.user.rnr.mg.dto.MgRnrInfoDTO;
import com.cusc.nirvana.user.rnr.mg.dto.MgVehicleCompanyDTO;
import com.cusc.nirvana.user.rnr.openapi.dto.QueryRnAuthDTO;
import com.cusc.nirvana.user.rnr.openapi.dto.VinDTO;
import com.cusc.nirvana.user.rnr.openapi.dto.response.RnAuthInfoRsp;
import com.cusc.nirvana.user.rnr.openapi.dto.response.RnAuthStateRsp;
import com.cusc.nirvana.user.rnr.openapi.service.IRnrAuthSearchService;
import com.cusc.nirvana.user.util.DateUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author yubo
 * @since 2022-06-09 11:05
 */
@Service
public class RnrAuthSearchServiceImpl implements IRnrAuthSearchService {

    @Autowired
    private MgRnrCardInfoClient mgRnrCardInfoClient;

    @Autowired
    private MgRnrInfoClient mgRnrInfoClient;

    @Autowired
    private MgVehicleCompanyClient mgVehicleCompanyClient;

    @Override
    public Response<List<RnAuthInfoRsp>> getRnrInfoByVin(VinDTO vinDTO) {
        MgRnrCardInfoDTO bean = new MgRnrCardInfoDTO();
        bean.setIotId(vinDTO.getVin());
        List<Integer> bizTypeList = new ArrayList<>();
        bizTypeList.add(RnrBizzTypeEnum.Bind.getCode());
        bizTypeList.add(RnrBizzTypeEnum.INFO_CHANGE.getCode());
        bizTypeList.add(RnrBizzTypeEnum.ChangeBinding.getCode());
        bizTypeList.add(RnrBizzTypeEnum.REPEAT_BIND.getCode());
        bean.setBizTypeList(bizTypeList);
        //todo 也要返回自然人的信息
        Response<List<MgRnrCardInfoDTO>> mgRsp = mgRnrCardInfoClient.queryByList(bean);
        if (mgRsp.isSuccess()) {
            List<MgRnrCardInfoDTO> cardInfoDTOS = mgRsp.getData();
            List<RnAuthInfoRsp> rnAuthInfoRspList = new ArrayList<>();
            if (!CollectionUtils.isEmpty(cardInfoDTOS)) {
                //根据createTime排序 取最新的实名状态信息
                MgRnrCardInfoDTO lastestRnrInfo = cardInfoDTOS.get(0);
                MgRnrInfoDTO mgRnrInfoDTO = new MgRnrInfoDTO();
                mgRnrInfoDTO.setUuid(lastestRnrInfo.getRnrId());
                Response<MgRnrInfoDTO> response = mgRnrInfoClient.getByUuid(mgRnrInfoDTO);

                RnAuthInfoRsp authInfoRsp = new RnAuthInfoRsp();
                BeanUtils.copyProperties(lastestRnrInfo, authInfoRsp);

                authInfoRsp.setRnrStatus(RnrStatus.getEnumByCode(lastestRnrInfo.getRnrStatus()).getComment());
                if (response.isSuccess() && null != response.getData()) {
                    BeanUtils.copyProperties(response.getData(), authInfoRsp);
                    authInfoRsp.setIsCompany(String.valueOf(response.getData().getIsCompany()));

                    if (1 == response.getData().getIsCompany()) {
                        MgVehicleCompanyDTO mgVehicleCompanyDTO = new MgVehicleCompanyDTO();
                        mgVehicleCompanyDTO.setUuid(response.getData().getRnrCompanyId());
                        Response<MgVehicleCompanyDTO> mgVehicleCompanyDTOResponse = mgVehicleCompanyClient.getByUuid(mgVehicleCompanyDTO);
                        if (mgVehicleCompanyDTOResponse.isSuccess() && null != mgVehicleCompanyDTOResponse.getData()) {
                            authInfoRsp.setCompanyName(mgVehicleCompanyDTOResponse.getData().getCompanyName());
                        }
                    }
                }
                rnAuthInfoRspList.add(authInfoRsp);

            }
            return Response.createSuccess("查询实名信息成功", rnAuthInfoRspList);
        } else {
            return Response.createError("查询实名信息失败:error->" + mgRsp.getMsg());
        }
    }


    @Override
    public Response<RnAuthStateRsp> getRnAuthState(QueryRnAuthDTO queryRnAuthDTO) {
        MgRnrCardInfoDTO cardInfoDTO = new MgRnrCardInfoDTO();
        if (StringUtils.isEmpty(queryRnAuthDTO.getVin()) && CollectionUtils.isEmpty(queryRnAuthDTO.getIccidList())) {
            return Response.createError("vin和iccid均为空,无法查询实名状态");
        }
        cardInfoDTO.setIotId(queryRnAuthDTO.getVin());
        List<Integer> bizTypeList = new ArrayList<>();
        bizTypeList.add(RnrBizzTypeEnum.Bind.getCode());
        bizTypeList.add(RnrBizzTypeEnum.INFO_CHANGE.getCode());
        bizTypeList.add(RnrBizzTypeEnum.ChangeBinding.getCode());
        bizTypeList.add(RnrBizzTypeEnum.REPEAT_BIND.getCode());
        cardInfoDTO.setBizTypeList(bizTypeList);
        cardInfoDTO.setIccidList(queryRnAuthDTO.getIccidList());
        Response<List<MgRnrCardInfoDTO>> mgRsp = mgRnrCardInfoClient.queryByList(cardInfoDTO);
        if (mgRsp.isSuccess()) {
            List<MgRnrCardInfoDTO> cardInfoDTOS = mgRsp.getData();
            RnAuthStateRsp authStateRsp = new RnAuthStateRsp();
            if (!CollectionUtils.isEmpty(cardInfoDTOS)) {
                Map<String, String> stateMap = new HashMap<>();
                Map<String, MgRnrCardInfoDTO> cardInfoDTOMap = new HashMap<>();
                //根据createTime排序 取最新的实名状态信息
                for (MgRnrCardInfoDTO rnrCardInfoDTO : cardInfoDTOS) {
                    String iccid = rnrCardInfoDTO.getIccid();
                    MgRnrCardInfoDTO infoDTO = cardInfoDTOMap.get(iccid);
                    if (null != infoDTO) {
                        if (DateUtils.compareDate(rnrCardInfoDTO.getCreateTime(), infoDTO.getCreateTime()) == 1) {
                            cardInfoDTOMap.put(iccid, rnrCardInfoDTO);
                        }
                    } else {
                        cardInfoDTOMap.put(iccid, rnrCardInfoDTO);
                    }
                }
                if (MapUtils.isNotEmpty(cardInfoDTOMap)) {
                    cardInfoDTOMap.values().forEach(lastestRnrState -> stateMap.put(lastestRnrState.getIotId() + "-" + lastestRnrState.getIccid(), RnrStatus.getEnumByCode(lastestRnrState.getRnrStatus()).getComment()));
                }

                authStateRsp.setRnAuthStateMap(stateMap);
            }
            return Response.createSuccess("查询实名状态成功", authStateRsp);
        } else {
            return Response.createError("查询实名状态失败:error->" + mgRsp.getMsg());
        }
    }
}
