/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.cusc.nirvana.user.rnr.openapi.util;

import com.cusc.nirvana.user.rnr.openapi.common.Constants;
import com.google.common.collect.Maps;
import lombok.extern.slf4j.Slf4j;

import java.util.*;
import java.util.stream.Collectors;

/**
 * SignUtils.
 */
@Slf4j
public final class SignUtils {

    private static final SignUtils SIGN_UTILS = new SignUtils();

    private SignUtils() {
    }

    /**
     * getInstance.
     *
     * @return {@linkplain SignUtils}
     */
    public static SignUtils getInstance() {
        return SIGN_UTILS;
    }

    /**
     * acquired sign.
     *
     * @param signKey sign key
     * @param params  params
     * @return sign
     */
    public static String generateSign(final String signKey, final Map<String, String> params) {
        List<String> storedKeys = Arrays.stream(params.keySet()
                .toArray(new String[]{}))
                .sorted(Comparator.naturalOrder())
                .collect(Collectors.toList());
        final String sign = storedKeys.stream()
                .filter(key -> !Objects.equals(key, Constants.SIGN))
                .map(key -> String.join("", key, params.get(key)))
                .collect(Collectors.joining()).trim();
        System.out.println("signstr:"+sign);
        // TODO this is a risk for error charset coding with getBytes
        return HMACUtils.sign(sign, signKey, HMACUtils.Type.HmacSHA256).toUpperCase();
    }

    /**
     * isValid.
     *
     * @param sign    sign
     * @param params  params
     * @param signKey signKey
     * @return boolean
     */
    public boolean isValid(final String sign, final Map<String, String> params, final String signKey) {
        log.info("加密信息====={}",generateSign(signKey, params));
        return Objects.equals(sign, generateSign(signKey, params));
    }

    /**
     * Generate key string.
     *
     * @return the string
     */
    public String generateKey() {
        return UUID.randomUUID().toString().replaceAll("-", "").toUpperCase();
    }

    public static void main(String[] args) {
        Map<String, String> map = Maps.newHashMapWithExpectedSize(4);
        map.put(Constants.APP_ID, "5455909B7FF644168384498F5E89C055");
        map.put(Constants.NONCE_STR, "a2f6cc37063b4246858142bacd4ba23g");
        map.put(Constants.TIMESTAMP, "1654825095546");
        map.put(Constants.VERSION, "1.0.0");
        System.out.println("sign---->" + SignUtils.generateSign("460660B2004B49708B141B423E209B74", map));
    }
}
